package org.yaoqiang.bpmn.model.elements.activities;

import org.yaoqiang.bpmn.model.elements.XMLAttribute;
import org.yaoqiang.bpmn.model.elements.XMLElement;
import org.yaoqiang.bpmn.model.elements.XMLTextElement;
import org.yaoqiang.bpmn.model.elements.core.common.Expression;
import org.yaoqiang.bpmn.model.elements.data.DataInput;
import org.yaoqiang.bpmn.model.elements.data.DataOutput;

/**
 * MultiInstanceLoopCharacteristics
 * 
 * @author Shi Yaoqiang(shi_yaoqiang@yahoo.com)
 */
public class MultiInstanceLoopCharacteristics extends LoopCharacteristics {

	private static final long serialVersionUID = -4923611309825631627L;

	public MultiInstanceLoopCharacteristics(XMLElement parent) {
		super(parent, "multiInstanceLoopCharacteristics");
	}

	protected void fillStructure() {
		XMLAttribute attrIsSequential = new XMLAttribute(this, "isSequential", Boolean.FALSE.toString());
		XMLAttribute attrBehavior = new XMLAttribute(this, "behavior", MultiInstanceBehavior.All.toString());
		XMLAttribute attrOneBehaviorEventRef = new XMLAttribute(this, "oneBehaviorEventRef");
		XMLAttribute attrNoneBehaviorEventRef = new XMLAttribute(this, "noneBehaviorEventRef");
		Expression refLoopCardinality = new Expression(this, "loopCardinality");
		XMLTextElement refLoopDataInputRef = new XMLTextElement(this, "loopDataInputRef");
		XMLTextElement refLoopDataOutputRef = new XMLTextElement(this, "loopDataOutputRef");
		DataInput refInputDataItem = new DataInput(this, "inputDataItem");
		DataOutput refOutputDataItem = new DataOutput(this, "outputDataItem");
		ComplexBehaviorDefinitions refComplexBehaviorDefinitions = new ComplexBehaviorDefinitions(this);
		Expression refCompletionCondition = new Expression(this, "completionCondition");

		super.fillStructure();
		add(attrIsSequential);
		add(attrBehavior);
		add(attrOneBehaviorEventRef);
		add(attrNoneBehaviorEventRef);
		add(refLoopCardinality);
		add(refLoopDataInputRef);
		add(refLoopDataOutputRef);
		add(refInputDataItem);
		add(refOutputDataItem);
		add(refComplexBehaviorDefinitions);
		add(refCompletionCondition);
	}
	
	public final boolean isSequential() {
		return Boolean.parseBoolean(get("isSequential").toValue());
	}
	
	public final void setIsSequential(boolean isSequential) {
		set("isSequential", String.valueOf(isSequential));
	}
	
	enum MultiInstanceBehavior {

		None("None"), One("One"), All("All"), Complex("Complex");

		private String direction;

		MultiInstanceBehavior(String direction) {
			this.setDirection(direction);
		}

		public void setDirection(String direction) {
			this.direction = direction;
		}

		public String getDirection() {
			return direction;
		}
		
	}
}
